@@ -11,21 +11,27 @@ from .execute import (
     default_type_resolver,
     ExecutionContext,
     ExecutionResult,
+    FormattedExecutionResult,
     Middleware,
 )
-
+from .map_async_iterator import MapAsyncIterator
+from .subscribe import subscribe, create_source_event_stream
 from .middleware import MiddlewareManager
-
-from .values import get_directive_values
+from .values import get_directive_values, get_variable_values
 
 __all__ = [
+    "create_source_event_stream",
     "execute",
     "execute_sync",
     "default_field_resolver",
     "default_type_resolver",
+    "subscribe",
     "ExecutionContext",
     "ExecutionResult",
+    "FormattedExecutionResult",
+    "MapAsyncIterator",
     "Middleware",
     "MiddlewareManager",
     "get_directive_values",
+    "get_variable_values",
 ]
