@@ -1,10 +1,9 @@
-from functools import wraps
-from json import dumps
-from typing import Any, Callable, Collection, Optional
+from typing import Any, Collection, Optional
 
 from ..language.ast import Node, OperationType
-from .visitor import visit, Visitor
 from .block_string import print_block_string
+from .print_string import print_string
+from .visitor import visit, Visitor
 
 __all__ = ["print_ast"]
 
@@ -50,16 +49,6 @@ def print_ast(ast: Node) -> str:
     return visit(ast, PrintAstVisitor())
 
 
-def add_description(method: Callable[..., str]) -> Callable:
-    """Decorator adding the description to the output of a static visitor method."""
-
-    @wraps(method)
-    def wrapped(node: PrintedNode, *args: Any) -> str:
-        return join((node.description, method(node, *args)), "\n")
-
-    return wrapped
-
-
 class PrintAstVisitor(Visitor):
     @staticmethod
     def leave_name(node: PrintedNode, *_args: Any) -> str:
@@ -73,20 +62,22 @@ class PrintAstVisitor(Visitor):
 
     @staticmethod
     def leave_document(node: PrintedNode, *_args: Any) -> str:
-        return join(node.definitions, "\n\n") + "\n"
+        return join(node.definitions, "\n\n")
 
     @staticmethod
     def leave_operation_definition(node: PrintedNode, *_args: Any) -> str:
-        name, op, selection_set = node.name, node.operation, node.selection_set
         var_defs = wrap("(", join(node.variable_definitions, ", "), ")")
-        directives = join(node.directives, " ")
+        prefix = join(
+            (
+                node.operation.value,
+                join((node.name, var_defs)),
+                join(node.directives, " "),
+            ),
+            " ",
+        )
         # Anonymous queries with no directives or variable definitions can use the
         # query short form.
-        return (
-            join((op.value, join((name, var_defs)), directives, selection_set), " ")
-            if (name or directives or var_defs or op != OperationType.QUERY)
-            else selection_set
-        )
+        return ("" if prefix == "query" else prefix + " ") + node.selection_set
 
     @staticmethod
     def leave_variable_definition(node: PrintedNode, *_args: Any) -> str:
@@ -134,8 +125,7 @@ class PrintAstVisitor(Visitor):
 
     @staticmethod
     def leave_fragment_definition(node: PrintedNode, *_args: Any) -> str:
-        # Note: fragment variable definitions are experimental and may be changed or
-        # removed in the future.
+        # Note: fragment variable definitions are deprecated and will be removed in v3.3
         return (
             f"fragment {node.name}"
             f"{wrap('(', join(node.variable_definitions, ', '), ')')}"
@@ -155,10 +145,10 @@ class PrintAstVisitor(Visitor):
         return node.value
 
     @staticmethod
-    def leave_string_value(node: PrintedNode, key: str, *_args: Any) -> str:
+    def leave_string_value(node: PrintedNode, *_args: Any) -> str:
         if node.block:
-            return print_block_string(node.value, "" if key == "description" else "  ")
-        return dumps(node.value)
+            return print_block_string(node.value)
+        return print_string(node.value)
 
     @staticmethod
     def leave_boolean_value(node: PrintedNode, *_args: Any) -> str:
@@ -207,10 +197,14 @@ class PrintAstVisitor(Visitor):
     # Type System Definitions
 
     @staticmethod
-    @add_description
     def leave_schema_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(
-            ("schema", join(node.directives, " "), block(node.operation_types)), " "
+        return wrap("", node.description, "\n") + join(
+            (
+                "schema",
+                join(node.directives, " "),
+                block(node.operation_types),
+            ),
+            " ",
         )
 
     @staticmethod
@@ -218,14 +212,19 @@ class PrintAstVisitor(Visitor):
         return f"{node.operation.value}: {node.type}"
 
     @staticmethod
-    @add_description
     def leave_scalar_type_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(("scalar", node.name, join(node.directives, " ")), " ")
+        return wrap("", node.description, "\n") + join(
+            (
+                "scalar",
+                node.name,
+                join(node.directives, " "),
+            ),
+            " ",
+        )
 
     @staticmethod
-    @add_description
     def leave_object_type_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(
+        return wrap("", node.description, "\n") + join(
             (
                 "type",
                 node.name,
@@ -237,7 +236,6 @@ class PrintAstVisitor(Visitor):
         )
 
     @staticmethod
-    @add_description
     def leave_field_definition(node: PrintedNode, *_args: Any) -> str:
         args = node.arguments
         args = (
@@ -246,12 +244,14 @@ class PrintAstVisitor(Visitor):
             else wrap("(", join(args, ", "), ")")
         )
         directives = wrap(" ", join(node.directives, " "))
-        return f"{node.name}{args}: {node.type}{directives}"
+        return (
+            wrap("", node.description, "\n")
+            + f"{node.name}{args}: {node.type}{directives}"
+        )
 
     @staticmethod
-    @add_description
     def leave_input_value_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(
+        return wrap("", node.description, "\n") + join(
             (
                 f"{node.name}: {node.type}",
                 wrap("= ", node.default_value),
@@ -261,9 +261,8 @@ class PrintAstVisitor(Visitor):
         )
 
     @staticmethod
-    @add_description
     def leave_interface_type_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(
+        return wrap("", node.description, "\n") + join(
             (
                 "interface",
                 node.name,
@@ -275,39 +274,36 @@ class PrintAstVisitor(Visitor):
         )
 
     @staticmethod
-    @add_description
     def leave_union_type_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(
+        return wrap("", node.description, "\n") + join(
             (
                 "union",
                 node.name,
                 join(node.directives, " "),
-                "= " + join(node.types, " | ") if node.types else "",
+                wrap("= ", join(node.types, " | ")),
             ),
             " ",
         )
 
     @staticmethod
-    @add_description
     def leave_enum_type_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(
+        return wrap("", node.description, "\n") + join(
             ("enum", node.name, join(node.directives, " "), block(node.values)), " "
         )
 
     @staticmethod
-    @add_description
     def leave_enum_value_definition(node: PrintedNode, *_args: Any) -> str:
-        return join((node.name, join(node.directives, " ")), " ")
+        return wrap("", node.description, "\n") + join(
+            (node.name, join(node.directives, " ")), " "
+        )
 
     @staticmethod
-    @add_description
     def leave_input_object_type_definition(node: PrintedNode, *_args: Any) -> str:
-        return join(
+        return wrap("", node.description, "\n") + join(
             ("input", node.name, join(node.directives, " "), block(node.fields)), " "
         )
 
     @staticmethod
-    @add_description
     def leave_directive_definition(node: PrintedNode, *_args: Any) -> str:
         args = node.arguments
         args = (
@@ -317,7 +313,10 @@ class PrintAstVisitor(Visitor):
         )
         repeatable = " repeatable" if node.repeatable else ""
         locations = join(node.locations, " | ")
-        return f"directive @{node.name}{args}{repeatable} on {locations}"
+        return (
+            wrap("", node.description, "\n")
+            + f"directive @{node.name}{args}{repeatable} on {locations}"
+        )
 
     @staticmethod
     def leave_schema_extension(node: PrintedNode, *_args: Any) -> str:
@@ -363,7 +362,7 @@ class PrintAstVisitor(Visitor):
                 "extend union",
                 node.name,
                 join(node.directives, " "),
-                "= " + join(node.types, " | ") if node.types else "",
+                wrap("= ", join(node.types, " | ")),
             ),
             " ",
         )
