@@ -12,8 +12,13 @@ from .schema import (
     assert_schema,
     # GraphQL Schema definition
     GraphQLSchema,
+    # Keyword Args
+    GraphQLSchemaKwargs,
 )
 
+# Uphold the spec rules about naming.
+from .assert_name import assert_name, assert_enum_value_name
+
 from .definition import (
     # Predicates
     is_type,
@@ -56,6 +61,8 @@ from .definition import (
     # Un-modifiers
     get_nullable_type,
     get_named_type,
+    # Thunk handling
+    resolve_thunk,
     # Definitions
     GraphQLScalarType,
     GraphQLObjectType,
@@ -76,7 +83,11 @@ from .definition import (
     GraphQLWrappingType,
     GraphQLNullableType,
     GraphQLNamedType,
+    GraphQLNamedInputType,
+    GraphQLNamedOutputType,
     Thunk,
+    ThunkCollection,
+    ThunkMapping,
     GraphQLArgument,
     GraphQLArgumentMap,
     GraphQLEnumValue,
@@ -88,6 +99,18 @@ from .definition import (
     GraphQLScalarSerializer,
     GraphQLScalarValueParser,
     GraphQLScalarLiteralParser,
+    # Keyword Args
+    GraphQLArgumentKwargs,
+    GraphQLEnumTypeKwargs,
+    GraphQLEnumValueKwargs,
+    GraphQLFieldKwargs,
+    GraphQLInputFieldKwargs,
+    GraphQLInputObjectTypeKwargs,
+    GraphQLInterfaceTypeKwargs,
+    GraphQLNamedTypeKwargs,
+    GraphQLObjectTypeKwargs,
+    GraphQLScalarTypeKwargs,
+    GraphQLUnionTypeKwargs,
     # Resolvers
     GraphQLFieldResolver,
     GraphQLTypeResolver,
@@ -109,6 +132,8 @@ from .directives import (
     GraphQLSkipDirective,
     GraphQLDeprecatedDirective,
     GraphQLSpecifiedByDirective,
+    # Keyword Args
+    GraphQLDirectiveKwargs,
     # Constant Deprecation Reason
     DEFAULT_DEPRECATION_REASON,
 )
@@ -124,6 +149,9 @@ from .scalars import (
     GraphQLString,
     GraphQLBoolean,
     GraphQLID,
+    # Int boundaries constants
+    GRAPHQL_MAX_INT,
+    GRAPHQL_MIN_INT,
 )
 
 from .introspection import (
@@ -145,7 +173,10 @@ from .validate import validate_schema, assert_valid_schema
 __all__ = [
     "is_schema",
     "assert_schema",
+    "assert_name",
+    "assert_enum_value_name",
     "GraphQLSchema",
+    "GraphQLSchemaKwargs",
     "is_type",
     "is_scalar_type",
     "is_object_type",
@@ -184,6 +215,7 @@ __all__ = [
     "assert_named_type",
     "get_nullable_type",
     "get_named_type",
+    "resolve_thunk",
     "GraphQLScalarType",
     "GraphQLObjectType",
     "GraphQLInterfaceType",
@@ -203,7 +235,11 @@ __all__ = [
     "GraphQLWrappingType",
     "GraphQLNullableType",
     "GraphQLNamedType",
+    "GraphQLNamedInputType",
+    "GraphQLNamedOutputType",
     "Thunk",
+    "ThunkCollection",
+    "ThunkMapping",
     "GraphQLArgument",
     "GraphQLArgumentMap",
     "GraphQLEnumValue",
@@ -215,6 +251,17 @@ __all__ = [
     "GraphQLScalarSerializer",
     "GraphQLScalarValueParser",
     "GraphQLScalarLiteralParser",
+    "GraphQLArgumentKwargs",
+    "GraphQLEnumTypeKwargs",
+    "GraphQLEnumValueKwargs",
+    "GraphQLFieldKwargs",
+    "GraphQLInputFieldKwargs",
+    "GraphQLInputObjectTypeKwargs",
+    "GraphQLInterfaceTypeKwargs",
+    "GraphQLNamedTypeKwargs",
+    "GraphQLObjectTypeKwargs",
+    "GraphQLScalarTypeKwargs",
+    "GraphQLUnionTypeKwargs",
     "GraphQLFieldResolver",
     "GraphQLTypeResolver",
     "GraphQLIsTypeOfFn",
@@ -229,6 +276,7 @@ __all__ = [
     "GraphQLSkipDirective",
     "GraphQLDeprecatedDirective",
     "GraphQLSpecifiedByDirective",
+    "GraphQLDirectiveKwargs",
     "DEFAULT_DEPRECATION_REASON",
     "is_specified_scalar_type",
     "specified_scalar_types",
@@ -237,6 +285,8 @@ __all__ = [
     "GraphQLString",
     "GraphQLBoolean",
     "GraphQLID",
+    "GRAPHQL_MAX_INT",
+    "GRAPHQL_MIN_INT",
     "is_introspection_type",
     "introspection_types",
     "TypeKind",
